@@ -1,3 +1,9 @@
+"""
+testfixtures.comparison
+-----------------------
+"""
+
+from decimal import Decimal
 from difflib import unified_diff
 from functools import partial
 from pprint import pformat
@@ -19,6 +25,8 @@ def compare_simple(x, y, context):
         repr_x = repr(x)
         repr_y = repr(y)
         if repr_x == repr_y:
+            if type(x) is not type(y):
+                return compare_with_type(x, y, context)
             x_attrs = _extract_attrs(x)
             y_attrs = _extract_attrs(y)
             diff = _compare_mapping(x_attrs, y_attrs, context, x,
@@ -32,33 +40,45 @@ def compare_simple(x, y, context):
 
 
 def _extract_attrs(obj, ignore=None):
+    try:
+        attrs = vars(obj).copy()
+    except TypeError:
+        attrs = None
+    else:
+        if isinstance(obj, BaseException):
+            attrs['args'] = obj.args
+
     has_slots = getattr(obj, '__slots__', not_there) is not not_there
     if has_slots:
         slots = set()
         for cls in type(obj).__mro__:
             slots.update(getattr(cls, '__slots__', ()))
-        attrs = {}
+        if slots and attrs is None:
+            attrs = {}
         for n in slots:
             value = getattr(obj, n, not_there)
             if value is not not_there:
                 attrs[n] = value
-    else:
-        try:
-            attrs = vars(obj).copy()
-        except TypeError:
-            return None
-        else:
-            if isinstance(obj, BaseException):
-                attrs['args'] = obj.args
+
+    if attrs is None:
+        return None
+
     if ignore is not None:
-        if isinstance(ignore, dict):
-            ignore = ignore.get(type(obj), ())
         for attr in ignore:
             attrs.pop(attr, None)
     return attrs
 
 
-def compare_object(x, y, context):
+def _attrs_to_ignore(context, ignore_attributes, obj):
+    ignore = context.get_option('ignore_attributes', ())
+    if isinstance(ignore, dict):
+        ignore = ignore.get(type(obj), ())
+    ignore = set(ignore)
+    ignore.update(ignore_attributes)
+    return ignore
+
+
+def compare_object(x, y, context, ignore_attributes=()):
     """
     Compare the two supplied objects based on their type and attributes.
 
@@ -67,12 +87,17 @@ def compare_object(x, y, context):
        Either a sequence of strings containing attribute names to be ignored
        when comparing or a mapping of type to sequence of strings containing
        attribute names to be ignored when comparing that type.
+
+       This may be specified as either a parameter to this function or in the
+       ``context``. If specified in both, they will both apply with precedence
+       given to whatever is specified is specified as a parameter.
+       If specified as a parameter to this fucntion, it may only be a list of
+       strings.
     """
-    ignore_attributes = context.get_option('ignore_attributes', ())
-    if type(x) is not type(y) or isinstance(x, ClassType):
+    if type(x) is not type(y) or isinstance(x, (ClassType, type)):
         return compare_simple(x, y, context)
-    x_attrs = _extract_attrs(x, ignore_attributes)
-    y_attrs = _extract_attrs(y, ignore_attributes)
+    x_attrs = _extract_attrs(x, _attrs_to_ignore(context, ignore_attributes, x))
+    y_attrs = _extract_attrs(y, _attrs_to_ignore(context, ignore_attributes, y))
     if x_attrs is None or y_attrs is None or not (x_attrs and y_attrs):
         return compare_simple(x, y, context)
     if x_attrs != y_attrs:
@@ -341,11 +366,34 @@ def compare_bytes(x, y, context):
 def compare_call(x, y, context):
     if x == y:
         return
-    x_name, x_args, x_kw = x
-    y_name, y_args, y_kw = y
+
+    def extract(call):
+        try:
+            name, args, kwargs = call
+        except ValueError:
+            name = None
+            args, kwargs = call
+        return name, args, kwargs
+
+    x_name, x_args, x_kw = extract(x)
+    y_name, y_args, y_kw = extract(y)
+
     if x_name == y_name and x_args == y_args and x_kw == y_kw:
         return compare_call(getattr(x, parent_name), getattr(y, parent_name), context)
-    return compare_text(repr(x), repr(y), context)
+
+    if repr(x) != repr(y):
+        return compare_text(repr(x), repr(y), context)
+
+    different = (
+        context.different(x_name, y_name, ' function name') or
+        context.different(x_args, y_args, ' args') or
+        context.different(x_kw, y_kw, ' kw')
+    )
+    if not different:
+        return
+
+    return 'mock.call not as expected:'
+
 
 
 def compare_partial(x, y, context):
@@ -372,6 +420,7 @@ _registry = {
     Unicode: compare_text,
     int: compare_simple,
     float: compare_simple,
+    Decimal: compare_simple,
     GeneratorType: compare_generator,
     mock_call.__class__: compare_call,
     unittest_mock_call.__class__: compare_call,
@@ -450,6 +499,13 @@ class CompareContext(object):
         if actual is not not_there:
             possible.append(actual)
 
+        x = self.options.pop('x', not_there)
+        if x is not not_there:
+            possible.append(x)
+        y = self.options.pop('y', not_there)
+        if y is not not_there:
+            possible.append(y)
+
         if len(possible) != 2:
             message = 'Exactly two objects needed, you supplied:'
             if possible:
@@ -550,14 +606,17 @@ class CompareContext(object):
 
 def compare(*args, **kw):
     """
-    Compare the two arguments passed either positionally or using
-    explicit ``expected`` and ``actual`` keyword paramaters. An
-    :class:`AssertionError` will be raised if they are not the same.
-    The :class:`AssertionError` raised will attempt to provide
+    Compare two objects, raising an :class:`AssertionError` if they are not
+    the same. The :class:`AssertionError` raised will attempt to provide
     descriptions of the differences found.
 
+    The two objects to compare can be passed either positionally or using
+    explicit keyword arguments named ``x`` and ``y``, or ``expected`` and
+    ``actual``.
+
     Any other keyword parameters supplied will be passed to the functions
-    that end up doing the comparison. See the API documentation below
+    that end up doing the comparison. See the
+    :mod:`API documentation below <testfixtures.comparison>`
     for details of these.
 
     :param prefix: If provided, in the event of an :class:`AssertionError`
@@ -568,6 +627,18 @@ def compare(*args, **kw):
                    being raised, the suffix supplied will be appended to the
                    message in the :class:`AssertionError`.
 
+    :param x_label: If provided, in the event of an :class:`AssertionError`
+                    being raised, the object passed as the first positional
+                    argument, or ``x`` keyword argument, will be labelled
+                    with this string in the message in the
+                    :class:`AssertionError`.
+
+    :param x_label: If provided, in the event of an :class:`AssertionError`
+                    being raised, the object passed as the second positional
+                    argument, or ``y`` keyword argument, will be labelled
+                    with this string in the message in the
+                    :class:`AssertionError`.
+
     :param raises: If ``False``, the message that would be raised in the
                    :class:`AssertionError` will be returned instead of the
                    exception being raised.
@@ -669,15 +740,19 @@ class Comparison(object):
         if self.v is None:
             return True
 
+        remaining_keys = set(self.v.keys())
         if self.strict:
             v = _extract_attrs(other)
+            remaining_keys -= set(v.keys())
         else:
             v = {}
-            for k in self.v.keys():
-                try:
-                    v[k] = getattr(other, k)
-                except AttributeError:
-                    pass
+
+        while remaining_keys:
+            k = remaining_keys.pop()
+            try:
+                v[k] = getattr(other, k)
+            except AttributeError:
+                pass
 
         kw = {'x_label': 'Comparison', 'y_label': 'actual'}
         context = CompareContext(kw)
