@@ -1,4 +1,5 @@
 from datetime import date, datetime
+from decimal import Decimal
 
 from functools import partial
 
@@ -19,9 +20,9 @@ from testfixtures import (
 from testfixtures.compat import (
     class_type_name, exception_module, PY3, xrange,
     BytesLiteral, UnicodeLiteral,
-    PY2, PY_37_PLUS
+    PY2, PY_37_PLUS, ABC
 )
-from testfixtures.comparison import compare_sequence
+from testfixtures.comparison import compare_sequence, compare_object
 from unittest import TestCase
 
 hexaddr = compile('0x[0-9A-Fa-f]+')
@@ -36,31 +37,41 @@ marker = object()
 
 _compare = compare
 
+
+def check_raises(x=marker, y=marker, message=None, regex=None,
+                 compare=compare, **kw):
+    args = []
+    for value in x, y:
+        if value is not marker:
+            args.append(value)
+    for value in 'x', 'y':
+        explicit = 'explicit_{}'.format(value)
+        if explicit in kw:
+            kw[value] = kw[explicit]
+            del kw[explicit]
+    try:
+        compare(*args, **kw)
+    except Exception as e:
+        if not isinstance(e, AssertionError):  # pragma: no cover
+            raise
+        actual = hexsub(e.args[0])
+        if message is not None:
+            # handy for debugging, but can't be relied on for tests!
+            _compare(actual, expected=message, show_whitespace=True)
+            assert actual == message
+        else:
+            if not regex.match(actual):  # pragma: no cover
+                raise AssertionError(
+                    '%r did not match %r' % (actual, regex.pattern)
+                )
+    else:
+        raise AssertionError('No exception raised!')
+
+
 class CompareHelper(object):
 
-    def check_raises(self, x=marker, y=marker, message=None, regex=None,
-                     compare=compare, **kw):
-        args = []
-        for value in x, y:
-            if value is not marker:
-                args.append(value)
-        try:
-            compare(*args, **kw)
-        except Exception as e:
-            if not isinstance(e, AssertionError):  # pragma: no cover
-                raise
-            actual = hexsub(e.args[0])
-            if message is not None:
-                # handy for debugging, but can't be relied on for tests!
-                _compare(actual, expected=message, show_whitespace=True)
-                assert actual == message
-            else:
-                if not regex.match(actual):  # pragma: no cover
-                    raise AssertionError(
-                        '%r did not match %r' % (actual, regex.pattern)
-                    )
-        else:
-            raise AssertionError('No exception raised!')
+    def check_raises(self, *args, **kw):
+        check_raises(*args, **kw)
 
 
 class TestCompare(CompareHelper, TestCase):
@@ -84,6 +95,10 @@ class TestCompare(CompareHelper, TestCase):
     def test_number_different(self):
         self.check_raises(1, 2, '1 != 2')
 
+    def test_decimal_different(self):
+        self.check_raises(Decimal(1), Decimal(2),
+                          "Decimal('1') != Decimal('2')")
+
     def test_different_with_labels(self):
         self.check_raises(1, 2, '1 (expected) != 2 (actual)',
                           x_label='expected', y_label='actual')
@@ -291,6 +306,30 @@ class TestCompare(CompareHelper, TestCase):
             "[4]"
             )
 
+    def test_list_different_float(self):
+        self.check_raises(
+            [1, 2, 3.0], [1, 2, 4.0],
+            "sequence not as expected:\n\n"
+            "same:\n"
+            "[1, 2]\n\n"
+            "first:\n"
+            "[3.0]\n\n"
+            "second:\n"
+            "[4.0]"
+            )
+
+    def test_list_different_decimal(self):
+        self.check_raises(
+            [1, 2, Decimal(3)], [1, 2, Decimal(4)],
+            "sequence not as expected:\n\n"
+            "same:\n"
+            "[1, 2]\n\n"
+            "first:\n"
+            "[Decimal('3')]\n\n"
+            "second:\n"
+            "[Decimal('4')]"
+            )
+
     def test_list_totally_different(self):
         self.check_raises(
             [1], [2],
@@ -748,6 +787,34 @@ class TestCompare(CompareHelper, TestCase):
             pass
         self.check_raises(X, Y, expected)
 
+    def test_new_style_classes_same(self):
+        class X(object):
+            pass
+        compare(X, X)
+
+    def test_new_style_classes_different(self):
+        if PY3:
+            expected = (
+                "<class 'testfixtures.tests.test_compare.TestCompare."
+                "test_new_style_classes_different.<locals>.X'>"
+                " != "
+                "<class 'testfixtures.tests.test_compare.TestCompare."
+                "test_new_style_classes_different.<locals>.Y'>"
+                )
+        else:
+            expected = (
+                "<class 'testfixtures.tests.test_compare.X'>"
+                " != "
+                "<class 'testfixtures.tests.test_compare.Y'>"
+                )
+
+        class X(object):
+            pass
+
+        class Y(object):
+            pass
+        self.check_raises(X, Y, expected)
+
     def test_show_whitespace(self):
         # does nothing! ;-)
         self.check_raises(
@@ -1325,13 +1392,18 @@ b
                           message="'x' (expected) != 'y' (actual)")
 
     def test_explicit_both(self):
-        self.check_raises(message="'x' (expected) != 'y' (actual)",
-                          expected='x', actual='y')
+        self.check_raises(expected='x', actual='y',
+                          message="'x' (expected) != 'y' (actual)")
+
+    def test_implicit_and_labels(self):
+        self.check_raises('x', 'y',
+                          x_label='x_label', y_label='y_label',
+                          message="'x' (x_label) != 'y' (y_label)")
 
     def test_explicit_and_labels(self):
-        self.check_raises(message="'x' (x_label) != 'y' (y_label)",
-                          expected='x', actual='y',
-                          x_label='x_label', y_label='y_label')
+        self.check_raises(explicit_x='x', explicit_y='y',
+                          x_label='x_label', y_label='y_label',
+                          message="'x' (x_label) != 'y' (y_label)")
 
     def test_invalid_two_args_expected(self):
         with ShouldRaise(TypeError(
@@ -1447,8 +1519,8 @@ b
         compare(m.mock_calls, m.mock_calls, strict=True)
 
     def test_calls_different(self):
-        m1 =Mock()
-        m2 =Mock()
+        m1 = Mock()
+        m2 = Mock()
         m1.foo(1, 2, x=3, y=4)
         m2.bar(1, 3, x=7, y=4)
 
@@ -1472,45 +1544,73 @@ b
             "'call.bar(1, 3, x=7, y=4)'"
         )
 
-    def test_compare_arbitrary_nested_same(self):
-        compare(SampleClassA([SampleClassB()]),
-                SampleClassA([SampleClassB()]))
+    def test_call_args_different(self):
+        m = Mock()
+        m.foo(1)
 
-    def test_compare_different_vars(self):
-        obj1 = SampleClassB(1)
-        obj1.same = 42
-        obj1.foo = '1'
-        obj2 = SampleClassB(2)
-        obj2.same = 42
-        obj2.bar = '2'
         self.check_raises(
-            obj1, obj2,
-            "SampleClassB not as expected:\n"
-            "\n"
-            "attributes same:\n"
-            "['same']\n"
-            "\n"
-            'attributes in first but not second:\n'
-            "'foo': '1'\n"
-            "\n"
-            'attributes in second but not first:\n'
-            "'bar': '2'\n"
+            m.foo.call_args,
+            call(2),
+            "'call(1)' != 'call(2)'"
+        )
+
+    def test_calls_args_different_but_same_repr(self):
+        class Annoying(object):
+            def __init__(self, x):
+                self.x = x
+            def __repr__(self):
+                return '<annoying>'
+        m1 = Mock()
+        m2 = Mock()
+        m1.foo(Annoying(1))
+        m2.foo(Annoying(3))
+
+        self.check_raises(
+            m1.mock_calls,
+            m2.mock_calls,
+            'sequence not as expected:\n'
             '\n'
-            'attributes differ:\n'
-            "'args': (1,) != (2,)\n"
+            'same:\n'
+            '[]\n'
             '\n'
-            "While comparing .args: sequence not as expected:\n"
+            'first:\n'
+            '[call.foo(<annoying>)]\n'
+            '\n'
+            'second:\n'
+            '[call.foo(<annoying>)]\n'
+            '\n'
+            'While comparing [0]: mock.call not as expected:\n'
+            '\n'
+            'While comparing [0] args: sequence not as expected:\n'
             '\n'
             'same:\n'
             '()\n'
             '\n'
             'first:\n'
-            '(1,)\n'
+            '(<annoying>,)\n'
             '\n'
             'second:\n'
-            '(2,)'
+            '(<annoying>,)\n'
+            '\n'
+            'While comparing [0] args[0]: Annoying not as expected:\n'
+            '\n'
+            'attributes differ:\n'
+            "'x': 1 != 3"
         )
 
+    def test_calls_nested_equal_sub_attributes(self):
+        class Annoying(object):
+            def __init__(self, x):
+                self.x = x
+            def __repr__(self):
+                return '<annoying>'
+        m1 = Mock()
+        m2 = Mock()
+        m1.foo(x=[Annoying(1)])
+        m2.foo(x=[Annoying(1)])
+
+        compare(m1.mock_calls, m2.mock_calls)
+
     def test_compare_arbitrary_nested_diff(self):
         class OurClass:
             def __init__(self, *args):
@@ -1619,6 +1719,26 @@ b
 
         compare(Child(1), Child(1))
 
+    def test_slots_and_attrs(self):
+
+        class Parent(object):
+            __slots__ = ('a',)
+
+        class Child(Parent):
+            def __init__(self, a, b):
+                self.a = a
+                self.b = b
+
+        self.check_raises(Child(1, 2), Child(1, 3), message=(
+            'Child not as expected:\n'
+            '\n'
+            'attributes same:\n'
+            "['a']\n"
+            '\n'
+            'attributes differ:\n'
+            "'b': 2 != 3"
+        ))
+
     def test_partial_callable_different(self):
 
         def foo(x): pass
@@ -1719,6 +1839,21 @@ b
             message="Both expected and actual appear as 'Wut', but are not equal!"
         )
 
+    def test_string_with_slotted(self):
+
+        class Slotted(object):
+            __slots__ = ['foo']
+            def __init__(self, foo):
+                self.foo = foo
+            def __repr__(self):
+                return repr(self.foo)
+
+        self.check_raises(
+            'foo',
+            Slotted('foo'),
+            "'foo' (%s) != 'foo' (%s)" % (repr(str), repr(Slotted))
+        )
+
     def test_not_recursive(self):
         self.check_raises(
             {1: 'foo', 2: 'foo'},
@@ -1772,3 +1907,85 @@ class TestIgnore(CompareHelper):
             "'id': 1 != 2",
             ignore_attributes=ignore
         )
+
+
+class TestCompareObject(object):
+
+    class Thing(object):
+        def __init__(self, **kw):
+            for k, v in kw.items():
+                setattr(self, k, v)
+
+    def test_ignore(self):
+        def compare_thing(x, y, context):
+            return compare_object(x, y, context, ignore_attributes=['y'])
+        compare(self.Thing(x=1, y=2), self.Thing(x=1, y=3),
+                comparers={self.Thing: compare_thing})
+
+    def test_ignore_dict_context_list_param(self):
+        def compare_thing(x, y, context):
+            return compare_object(x, y, context, ignore_attributes=['y'])
+        compare(self.Thing(x=1, y=2, z=3), self.Thing(x=1, y=4, z=5),
+                comparers={self.Thing: compare_thing},
+                ignore_attributes={self.Thing: ['z']})
+
+    def test_ignore_list_context_list_param(self):
+        def compare_thing(x, y, context):
+            return compare_object(x, y, context, ignore_attributes=['y'])
+        compare(self.Thing(x=1, y=2, z=3), self.Thing(x=1, y=4, z=5),
+                comparers={self.Thing: compare_thing},
+                ignore_attributes=['z'])
+
+
+class BaseClass(ABC):
+    pass
+
+
+class MyDerivedClass(BaseClass):
+
+    def __init__(self, thing):
+        self.thing = thing
+
+
+class ConcreteBaseClass(object): pass
+
+
+class ConcreteDerivedClass(ConcreteBaseClass):
+
+    def __init__(self, thing):
+        self.thing = thing
+
+
+class TestBaseClasses(CompareHelper):
+
+    def test_abc_equal(self):
+        thing1 = MyDerivedClass(1)
+        thing2 = MyDerivedClass(1)
+
+        compare(thing1, thing2)
+
+    def test_abc_unequal(self):
+        thing1 = MyDerivedClass(1)
+        thing2 = MyDerivedClass(2)
+
+        self.check_raises(thing1, thing2, message=(
+            "MyDerivedClass not as expected:\n\n"
+            "attributes differ:\n"
+            "'thing': 1 != 2"
+        ))
+
+    def test_concrete_equal(self):
+        thing1 = ConcreteDerivedClass(1)
+        thing2 = ConcreteDerivedClass(1)
+
+        compare(thing1, thing2)
+
+    def test_concrete_unequal(self):
+        thing1 = ConcreteDerivedClass(1)
+        thing2 = ConcreteDerivedClass(2)
+
+        self.check_raises(thing1, thing2, message=(
+            "ConcreteDerivedClass not as expected:\n\n"
+            "attributes differ:\n"
+            "'thing': 1 != 2"
+        ))
